﻿//
//       PCI-9018 class file for C#
//
//       Revision History:
//       2019-10  create for PCI-9018
//       2021-3   add constant definitions;
//                update namespace.
using System.Runtime.InteropServices;

namespace SGMotion
{
    public class PCI9018
    {
        public const int MOT_MAX_AXIS = 8;  //number of axes in one device
        public const int MOT_MAX_DIO = 16;  //number of DIs and DOs in one device

        public const int DIO_ON = 1;        //DI/DO turn-on
        public const int DIO_OFF = 0;       //DI/DO turn-off

        public enum PN_NUMBER
        {
            PN_OutputMode = 1000,           //pulse output mode, 0 for PUL/DIR, 1 for CW/CCW

            PN_InvertOutputDirection = 1004, //invert direction output, non-zero for enable, zero for disable
            PN_InputMode,                   //pulse input mode, 0 for PUL/DIR, 1 for 4xAB
            PN_InvertInputDirection,        //invert input counter(feedback counter) direction, non-zero for enable, zero for disable
            PN_PosLimitSwitchType,          //hardware limit switch type, zero for normal open, non-zero for normal close
            PN_PosLimitSwitchEnable,        //enable positive hard limit input, non-zero for enable, zero for disable
            PN_NegLimitSwitchType,          //reserved
            PN_NegLimitSwitchEnable,        //enable negative hard limit input, non-zero for enable, zero for disable
            PN_HomeSwitchType,              //home switch type, zero for normal open, non-zero for normal close
            PN_HomeSwitchEnable,            //reserved
            PN_IndexActiveLevel,            //index active level, zero for low level, non-zero for high level
            PN_AmpFaultType,                //servo fault input type,  zero for normal open, non-zero for normal close
            PN_AmpFaultEnable,              //enable amplifier fault input
            PN_AbortSwitchType,             //abort input type, zero for normal open, non-zero for normal close
            PN_AbortSwitchEnable            //enable abort input, non-zero for enable, zero for disable
        }

        [StructLayoutAttribute(LayoutKind.Sequential)]
        public struct MOT_AXIS_STAT
        {
            public double cmdPos;      //current command position
            public double cmdVel;      //current command velocity
            public double cmdAcc;      //reserved
            public double actPos;      //current feedback position
            public double actVel;      //reserved
            public double actAcc;      //reserved
            public double reserved1;   //reserved
            public double reserved2;   //reserved
            public int homeSwitch;     //non-zero means home switch is tripped
            public int negHardLimit;   //non-zero means negative hard limit tripped
            public int posHardLimit;   //non-zero means positive hard limit tripped
            public int fault;          //non-zero means servo amplifier fault
            public int abort;          //non-zero means abort is active
            public int minSoftLimit;   //reserved
            public int maxSoftLimit;   //reserved
            public int inpos;          //non-zero means axis stops
            public int enabled;        //non-zero means servo_on is turned on
            public int homing;         //reserved
            public int homed;          //reserved
            public int error;          //non-zero means error(exception encounterd in last move)
        }

        [StructLayoutAttribute(LayoutKind.Sequential)]
        public struct MOT_CMP_STAT
        {
            public int queFull;         //nonzero means reference queue full
            public int queEmpty;        //nonzero means reference queue empty
            public int queItem;         //queued items
            public int curRef;          //current reference
            public int matchCount;      //match count
            public int reserved1;       //reserved
            public int reserved2;       //reserved
            public int reserved3;       //reserved
        }

        [DllImport("PCI9018.dll")]
        public static extern int p9018_open(ref int count, int[] ID);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_close();

        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_cont_move(int axis, double vel);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_incr_move(int axis, int incr);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_abs_move(int axis, int pos);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_home(int axis, double vel, int use_index);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_abort(int axis);

        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_set_cmd_pos(int axis, int pos);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_set_act_pos(int axis, int pos);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_set_servo_on(int axis, int on);

        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_set_profile(int axis, double min_vel, double max_vel, double accel, double decel, double jerk);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_set_parameter(int axis, int param_num, int value);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_set_real_parameter(int axis, int param_num, double value);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_get_status(int axis, ref MOT_AXIS_STAT stat);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_axis_get_status_ex(int axis, int[] buff, int cnt);

        [DllImport("PCI9018.dll")]
        public static extern int p9018_cmp_enable(int card_no, int index, int enable, int active_level, int ref_source, int length);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_cmp_set_ref(int card_no, int index, int pos);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_cmp_get_status(int card_no, int index, ref MOT_CMP_STAT stat);

        [DllImport("PCI9018.dll")]
        public static extern int p9018_motion_set_dout(int card_no, int index, int value);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_motion_set_dout_ex(int card_no, uint value);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_motion_get_dout(int card_no, int index, ref int value);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_motion_get_dout_ex(int card_no, ref uint value);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_motion_get_din(int card_no, int index, ref int value);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_motion_get_din_ex(int card_no, ref uint value);
        [DllImport("PCI9018.dll")]
        public static extern int p9018_motion_get_info(int card_no, uint[] buff, int count);
    }
}