#ifndef __PCI9018_H__
#define __PCI9018_H__

#include <WinDef.h>     //UINT

#define PCI9018_API __declspec(dllimport) int __stdcall

#define DIO_ON      1               //DI/DO turn-on
#define DIO_OFF     0               //DI/DO turn-off

typedef enum _PN_NUMBER{
    PN_OutputMode = 1000,           //pulse output mode, 0 for PUL/DIR, 1 for CW/CCW

    PN_InvertOutputDirection = 1004, //invert output direction, non-zero for enable, zero for disable
    PN_InputMode,                   //pulse input mode, 0 for PUL/DIR, 1 for 4xAB
    PN_InvertInputDirection,        //invert input counter(feedback counter) direction, non-zero for enable, zero for disable
    PN_PosLimitSwitchType,          //hardware limit switch type, zero for normal open, non-zero for normal close
    PN_PosLimitSwitchEnable,        //enable positive hard limit input, non-zero for enable, zero for disable
    PN_NegLimitSwitchType,          //reserved
    PN_NegLimitSwitchEnable,        //enable negative hard limit input, non-zero for enable, zero for disable
    PN_HomeSwitchType,              //home switch type, zero for normal open, non-zero for normal close
    PN_HomeSwitchEnable,            //reserved
    PN_IndexActiveLevel,            //index active level, zero for low level, non-zero for high level
    PN_AmpFaultType,                //servo fault input type,  zero for normal open, non-zero for normal close
    PN_AmpFaultEnable,              //enable servo fault input, non-zero for enable, zero for disable
    PN_AbortSwitchType,             //abort input type, zero for normal open, non-zero for normal close
    PN_AbortSwitchEnable            //enable abort input, non-zero for enable, zero for disable
}PN_NUMBER;

//  Data structures (set packing to 4-byte).
#pragma pack(push,4)
typedef struct _MOT_AXIS_STAT{
    double          cmdPos;            //current command position
    double          cmdVel;            //current command velocity
    double          cmdAcc;            //reserved
    double          actPos;            //current feedback position
    double          actVel;            //reserved
    double          actAcc;            //reserved
    double          reserved1;         //reserved
    double          reserved2;         //reserved
    int             homeSwitch;         //non-zero means home switch is tripped
    int             negHardLimit;       //non-zero means positive hard limit tripped
    int             posHardLimit;       //non-zero means negative hard limit tripped
    int             fault;              //non-zero means servo fault
    int             abort;              //non-zero means abort is active
    int             minSoftLimit;       //reserved
    int             maxSoftLimit;       //reserved
    int             inpos;              //non-zero means axis stops
    int             enabled;            //non-zero means servo_on is turned on
    int             homing;             //reserved
    int             homed;              //reserved
    int             error;              //non-zero means error(exception encounterd in last move)
}MOT_AXIS_STAT;

typedef struct _MOT_CMP_STAT{
    int             queFull;            //nonzero means reference queue full
    int             queEmpty;           //nonzero means reference queue empty
    int             queItem;            //queued items
    int             curRef;             //current reference
    int             matchCount;         //match count
    int             reserved[3];        //reserved
}MOT_CMP_STAT;

//  Restore default packing
#pragma pack(pop)

#ifdef __cplusplus
extern "C" {
#endif
PCI9018_API p9018_open(int* pCount, int* pId);
PCI9018_API p9018_close();

PCI9018_API p9018_axis_cont_move(int axis, double vel);
PCI9018_API p9018_axis_incr_move(int axis, int incr);
PCI9018_API p9018_axis_abs_move(int axis, int pos);
PCI9018_API p9018_axis_home(int axis, double vel, int use_index);
PCI9018_API p9018_axis_abort(int axis);

PCI9018_API p9018_axis_set_cmd_pos(int axis, int pos);
PCI9018_API p9018_axis_set_act_pos(int axis, int pos);
PCI9018_API p9018_axis_set_servo_on(int axis, int on);
PCI9018_API p9018_axis_set_profile(int axis, double min_vel, double max_vel, double accel, double decel, double jerk);
PCI9018_API p9018_axis_set_parameter(int axis, int param_num, int value);
PCI9018_API p9018_axis_set_real_parameter(int axis, int param_num, double value);

PCI9018_API p9018_axis_get_status(int axis, MOT_AXIS_STAT * pStat);
PCI9018_API p9018_axis_get_status_ex(int axis, int* buff, int cnt);

PCI9018_API p9018_cmp_enable(int card_no, int index, int enable, int active_level, int ref_source, int length);
PCI9018_API p9018_cmp_set_ref(int card_no, int index, int pos);
PCI9018_API p9018_cmp_get_status(int card_no, int index, MOT_CMP_STAT* pStatus);

PCI9018_API p9018_motion_set_dout(int card_no, int index, int value);
PCI9018_API p9018_motion_set_dout_ex(int card_no, UINT value);
PCI9018_API p9018_motion_get_dout(int card_no, int index, int* pValue);
PCI9018_API p9018_motion_get_dout_ex(int card_no, UINT* pValue);
PCI9018_API p9018_motion_get_din(int card_no, int index, int* pValue);
PCI9018_API p9018_motion_get_din_ex(int card_no, UINT* pValue);

PCI9018_API p9018_motion_get_info(int card_no, UINT* buff, int count);
#ifdef __cplusplus
}
#endif
#endif