using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SGMotion;

namespace McDemo10
{
    public partial class Form1 : Form
    {
        const int AXES_IN_DEV = 8;

        int axis;
        int m_axis;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int rc;
            int cardCount = 0;
            int[] cardIDs = new int[16];

            rc = PCI9018.p9018_open(ref cardCount, cardIDs);
            if (rc != 0)
            {
                MessageBox.Show("Fails to open device", "Error");
                Application.Exit();
            }
            if (cardCount < 1)
            {
                MessageBox.Show("Find No device", "Error");
                Application.Exit();
            }
            else
            {
                for (int i = 0; i < cardCount; i++)
                {
                    for (int j = 0; j < AXES_IN_DEV; j++)
                    {
                        axis = cardIDs[i] * AXES_IN_DEV + j;
                        ComboAxisSel.Items.Add(axis.ToString());
                    }
                }
                ComboAxisSel.SelectedIndex = 0;
                m_axis = cardIDs[0] * AXES_IN_DEV + 0;
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            PCI9018.p9018_close();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            double start_vel, max_vel, acc, dec;
            int dist;
            int rc = 0;
            start_vel = Convert.ToDouble(txtAxisMinVel.Text);
            max_vel = Convert.ToDouble(txtAxisMaxVel.Text);
            acc = Convert.ToDouble(txtAxisAccel.Text);
            dec = Convert.ToDouble(txtAxisDecel.Text);
            dist = Convert.ToInt32(txtAxisTarget.Text);

            PCI9018.p9018_axis_set_profile(m_axis, start_vel, max_vel, acc, dec, acc * 1000.0);

            if (radAxisPositioningMove.Checked)
            {
                rc = PCI9018.p9018_axis_abs_move(m_axis, dist);

            }
            else if (radAxisContMovePos.Checked)
            {
                rc = PCI9018.p9018_axis_cont_move(m_axis, Math.Abs(max_vel));
            }
            else if (radAxisContMoveNeg.Checked)
            {
                rc = PCI9018.p9018_axis_cont_move(m_axis, -Math.Abs(max_vel));
            }
            else if (radAxisSearchHomePos.Checked)
            {
                rc = PCI9018.p9018_axis_home(m_axis, Math.Abs(max_vel), 0);
            }
            else
            {
                rc = PCI9018.p9018_axis_home(m_axis, -Math.Abs(max_vel), 0);
            }

            if (rc != 0)
                MessageBox.Show("Fails to send command, return code:" + rc.ToString(), "Error");
        }

        private void timer1_Tick(object sender, EventArgs e)
        {

            PCI9018.MOT_AXIS_STAT stat = new PCI9018.MOT_AXIS_STAT();
            int rc;

            rc = PCI9018.p9018_axis_get_status(m_axis, ref stat);
            if (rc == 0)
            {
                txtAxisCmdVel.Text = stat.cmdVel.ToString("f1");
                txtAxisStatus.Text = (stat.inpos == 0) ? "Y" : "N";
                txtAxisCmdPos.Text = stat.cmdPos.ToString("f1");
                txtAxisFbPos.Text = stat.actPos.ToString("f1");

                //PEL
                txtAxisPel.Text = (stat.posHardLimit != 0) ? "Y" : "N";
                //MEL
                txtAxisMel.Text = (stat.negHardLimit != 0) ? "Y" : "N";
                //Home
                txtAxisHome.Text = (stat.homeSwitch != 0) ? "Y" : "N";
                //Fault
                txtAxisFault.Text = (stat.fault != 0) ? "Y" : "N";
            }
        }

        private void m_ctlAxisSel_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ComboAxisSel.SelectedIndex < 0)
                return;

            m_axis = Convert.ToInt32(ComboAxisSel.SelectedItem);
        }

        private void SdStop_Click(object sender, EventArgs e)
        {
            int rc = 0;

            rc = PCI9018.p9018_axis_abort(m_axis);
            if (rc != 0)
            {
                MessageBox.Show("Fails to stop axis, return code:" + rc.ToString(), "Error");
            }
        }
    }
}