Option Strict On
Option Explicit On

Public Class Form1
    Private m_axis As Integer
    Private Const AXES_IN_DEV As Integer = 8

    Private Sub ButtonRun_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAxisRun.Click
        Dim pos As Integer
        Dim start_vel, max_vel, accel, decel As Double
        Dim type As Integer
        Dim rc As Integer

        If (ComboBoxMotionType.SelectedIndex < 0) Then
            type = 0
        Else
            type = ComboBoxMotionType.SelectedIndex
        End If

        pos = CInt(TextBoxTargetPos.Text)
        start_vel = CDbl(TextBoxStartVel.Text)
        max_vel = CDbl(TextBoxMaxVel.Text)
        accel = CDbl(TextBoxAccel.Text)
        decel = CDbl(TextBoxDecel.Text)


        p9018_axis_set_profile(m_axis, start_vel, max_vel, accel, decel, accel * 1000.0)

        Select Case type
            Case 0
                'abs move
                rc = p9018_axis_abs_move(m_axis, pos)

            Case 1
                'incr move
                rc = p9018_axis_incr_move(m_axis, pos)

            Case 2
                'cont move +
                rc = p9018_axis_cont_move(m_axis, Math.Abs(max_vel))

            Case 3
                'cont move -
                rc = p9018_axis_cont_move(m_axis, -Math.Abs(max_vel))

            Case 4
                'home search+
                rc = p9018_axis_home(m_axis, Math.Abs(max_vel), 0)

            Case 5
                'home search -
                rc = p9018_axis_home(m_axis, -Math.Abs(max_vel), 0)

            Case 6
                'clear pos.
                rc = p9018_axis_set_cmd_pos(m_axis, 0)
                rc = p9018_axis_set_act_pos(m_axis, 0)

        End Select

        If rc <> 0 Then
            MsgBox("Drive axis fail, return code: " + rc.ToString())
        End If
    End Sub

    Private Sub ButtonStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAxisStop.Click
        Dim rc As Integer

        rc = p9018_axis_abort(m_axis)
        If rc <> 0 Then
            MsgBox("Stop axis fail, return code: " + rc.ToString())
        End If
    End Sub

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
    End Sub

    Protected Overrides Sub Finalize()
        MyBase.Finalize()

        p9018_close()
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        Dim axis As Integer
        Dim axisStat As MOT_AXIS_STAT
        Dim rc As Integer

        axis = m_axis
        rc = p9018_axis_get_status(m_axis, axisStat)
        If rc <> 0 Then
            Return
        End If

        TextBoxCurVel.Text = axisStat.cmdVel.ToString("f1")
        TextBoxCmdPos.Text = axisStat.cmdPos.ToString("f1")
        TextBoxEncPos.Text = axisStat.actPos.ToString("f1")

        TextBoxStatus.Text = If(axisStat.inpos = 0, "Y", "N")

        TextBoxPel.Text = If(axisStat.posHardLimit <> 0, "Y", "N")
        TxtAxisMel.Text = If(axisStat.negHardLimit <> 0, "Y", "N")
        TxtAxisHome.Text = If(axisStat.homeSwitch <> 0, "Y", "N")
        TxtAxisFault.Text = If(axisStat.fault <> 0, "Y", "N")
        TxtAxisAbort.Text = If(axisStat.abort <> 0, "Y", "N")
    End Sub

    Private Sub ComboBoxAxis_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBoxAxis.SelectedIndexChanged
        If ComboBoxAxis.SelectedIndex < 0 Then
            Return
        End If

        m_axis = CInt(ComboBoxAxis.SelectedItem)
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim cardCount As Integer
        Dim cardIDs(16) As Integer
        Dim rc As Integer
        Dim i, j As Integer
        Dim axis As Integer

        rc = p9018_open(cardCount, cardIDs)
        If rc <> 0 Then
            MsgBox("Init. fails")
            Application.Exit()
        End If
        If cardCount < 1 Then
            MsgBox("Find no card")
            Application.Exit()
        End If

        For i = 0 To cardCount - 1
            For j = 0 To AXES_IN_DEV - 1
                axis = cardIDs(i) * AXES_IN_DEV + j
                ComboBoxAxis.Items.Add(axis.ToString())
            Next
        Next

        m_axis = cardIDs(0) * AXES_IN_DEV + 0
        ComboBoxAxis.SelectedIndex = 0      'select first axis
        ComboBoxMotionType.SelectedIndex = 0
    End Sub
End Class
