'
'   Header File for PCI-9018
'
'   History:
'   2019/10/22   created for PCI-9018
'   2021/3/4     add constant definitions
'
Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices
Module PCI9018
#Region "Global constants"
    Public Const MOT_MAX_AXIS As Integer = 8  'number of axes in one device
    Public Const MOT_MAX_DIO As Integer = 16  'number of DIs and DOs in one device

    Public Const DIO_ON As Integer = 1       'DI/DO turn-On
    Public Const DIO_OFF As Integer = 0      'DI/DO turn-off

    Public Enum PN_NUMBER
        PN_OutputMode = 1000                 'pulse output mode, 0 for PUL/DIR, 1 for CW/CCW

        PN_InvertOutputDirection = 1004     'invert direction output, non-zero for enable, zero for disable
        PN_InputMode                        'pulse input mode, 0 for PUL/DIR, 1 for 4xAB
        PN_InvertInputDirection             'invert feedback counter counting direction, non-zero for enable, zero for disable
        PN_PosLimitSwitchType               'hardware limit switch type, zero for normal open, non-zero for normal close
        PN_PosLimitSwitchEnable             'enable positive hard limit input, non-zero for enable, zero for disable
        PN_NegLimitSwitchType               'reserved
        PN_NegLimitSwitchEnable             'enable negative hard limit input, non-zero for enable, zero for disable
        PN_HomeSwitchType                   'home switch type, zero for normal open, non-zero for normal close
        PN_HomeSwitchEnable                 'reserved
        PN_IndexActiveLevel                 'index active level, zero for low level, non-zero for high level
        PN_AmpFaultType                     'servo fault input type, zero for normal open, non-zero for normal close
        PN_AmpFaultEnable                   'enable servo fault input, non-zero for enable, zero for disable
        PN_AbortSwitchType                  'abort input type, zero for normal open, non-zero for normal close
        PN_AbortSwitchEnable                'enable abort input, non-zero for enable, zero for disable
    End Enum
#End Region

#Region "Structure Definitions"
    <StructLayoutAttribute(LayoutKind.Sequential, Pack:=4)> _
    Public Structure MOT_AXIS_STAT
        Public cmdPos As Double         'current command position
        Public cmdVel As Double         'current command velocity
        Public cmdAcc As Double         'reserved
        Public actPos As Double         'current feedback position
        Public actVel As Double         'reserved
        Public actAcc As Double         'reserved
        Public reserved1 As Double      'reserved
        Public reserved2 As Double      'reserved
        Public homeSwitch As Integer    'non-zero means home switch is tripped
        Public negHardLimit As Integer  'non-zero means negative hard limit tripped
        Public posHardLimit As Integer  'non-zero means positive hard limit tripped
        Public fault As Integer         'non-zero means servo fault
        Public abort As Integer         'non-zero means abort is active
        Public minSoftLimit As Integer  'reserved
        Public maxSoftLimit As Integer  'reserved
        Public inpos As Integer         'non-zero means axis stops
        Public enabled As Integer       'non-zero means servo_on is turned on
        Public homing As Integer        'reserved
        Public homed As Integer         'reserved
        Public err As Integer           'non-zero means error(exception encounterd in last move)
    End Structure

    <StructLayoutAttribute(LayoutKind.Sequential, Pack:=4)> _
    Public Structure MOT_CMP_STAT
        Public queFull As Integer       'nonzero means reference queue is full
        Public queEmpty As Integer      'nonzero means reference queue is empty
        Public queItem As Integer       'queued items
        Public curRef As Integer        'current reference
        Public matchCount As Integer    'match count
        Public reserved1 As Integer     'reserved
        Public reserved2 As Integer     'reserved
        Public reserved3 As Integer     'reserved
    End Structure
#End Region

#Region "Function Prototype"
    Declare Function p9018_open Lib "PCI9018.dll" (ByRef count As Integer, ByVal id() As Integer) As Integer
    Declare Function p9018_close Lib "PCI9018.dll" () As Integer

    Declare Function p9018_axis_cont_move Lib "PCI9018.dll" (ByVal axis As Integer, ByVal vel As Double) As Integer
    Declare Function p9018_axis_incr_move Lib "PCI9018.dll" (ByVal axis As Integer, ByVal incr As Integer) As Integer
    Declare Function p9018_axis_abs_move Lib "PCI9018.dll" (ByVal axis As Integer, ByVal pos As Integer) As Integer
    Declare Function p9018_axis_home Lib "PCI9018.dll" (ByVal axis As Integer, ByVal vel As Double, ByVal use_index As Integer) As Integer
    Declare Function p9018_axis_abort Lib "PCI9018.dll" (ByVal axis As Integer) As Integer

    Declare Function p9018_axis_set_cmd_pos Lib "PCI9018.dll" (ByVal axis As Integer, ByVal pos As Integer) As Integer
    Declare Function p9018_axis_set_act_pos Lib "PCI9018.dll" (ByVal axis As Integer, ByVal pos As Integer) As Integer
    Declare Function p9018_axis_set_servo_on Lib "PCI9018.dll" (ByVal axis As Integer, ByVal [on] As Integer) As Integer

    Declare Function p9018_axis_set_profile Lib "PCI9018.dll" (ByVal axis As Integer, ByVal min_vel As Double, ByVal max_vel As Double, ByVal accel As Double, ByVal decel As Double, ByVal jerk As Double) As Integer
    Declare Function p9018_axis_set_parameter Lib "PCI9018.dll" (ByVal axis As Integer, ByVal param_num As Integer, ByVal value As Integer) As Integer
    'Declare Function p9018_axis_set_real_parameter Lib "PCI9018.dll" (ByVal axis As Integer, ByVal param_num As Integer, ByVal value As Double) As Integer
    Declare Function p9018_axis_get_status Lib "PCI9018.dll" (ByVal axis As Integer, ByRef stat As MOT_AXIS_STAT) As Integer

    Declare Function p9018_cmp_enable Lib "PCI9018.dll" (ByVal card_no As Integer, ByVal index As Integer, ByVal enable As Integer, ByVal active_level As Integer, ByVal ref_source As Integer, ByVal length As Integer) As Integer
    Declare Function p9018_cmp_set_ref Lib "PCI9018.dll" (ByVal card_no As Integer, ByVal index As Integer, ByVal pos As Integer) As Integer
    Declare Function p9018_cmp_get_status Lib "PCI9018.dll" (ByVal card_no As Integer, ByVal index As Integer, ByRef stat As MOT_CMP_STAT) As Integer

    Declare Function p9018_motion_set_dout Lib "PCI9018.dll" (ByVal card_no As Integer, ByVal index As Integer, ByVal value As Integer) As Integer
    Declare Function p9018_motion_set_dout_ex Lib "PCI9018.dll" (ByVal card_no As Integer, ByVal value As UInteger) As Integer
    Declare Function p9018_motion_get_dout Lib "PCI9018.dll" (ByVal card_no As Integer, ByVal index As Integer, ByRef value As Integer) As Integer
    Declare Function p9018_motion_get_dout_ex Lib "PCI9018.dll" (ByVal card_no As Integer, ByRef value As UInteger) As Integer
    Declare Function p9018_motion_get_din Lib "PCI9018.dll" (ByVal card_no As Integer, ByVal index As Integer, ByRef value As Integer) As Integer
    Declare Function p9018_motion_get_din_ex Lib "PCI9018.dll" (ByVal card_no As Integer, ByRef value As UInteger) As Integer

    Declare Function p9018_motion_get_info Lib "PCI9018.dll" (ByVal card_no As Integer, ByVal buff As UInteger(), ByVal cnt As Integer) As Integer
#End Region
End Module