﻿'
'   PCI-9018调试软件(VB.NET)
'   Date: 2019/10
'

Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices
Imports System.Collections.Generic
Public Class Form1
    Public g_AppVersionStr As String = "1.0.0.0"    '应用程序版本号
    Public MOT_DIO_IN_DEV As Integer = 16       'DIOs in a device
    Public MOT_AXES_IN_DEV As Integer = 8       'axes in a device
    Public MOT_MAX_DEV As Integer = 16          'maximum devices supported by API

    Private m_din(MOT_DIO_IN_DEV - 1) As Integer
    Private m_dout(MOT_DIO_IN_DEV - 1) As Integer

    Private m_AxisJogPos1(MOT_AXES_IN_DEV * MOT_MAX_DEV - 1) As Integer
    Private m_AxisJogPos2(MOT_AXES_IN_DEV * MOT_MAX_DEV - 1) As Integer      'target2 for repeat mode
    Private m_AxisRepeatMove(MOT_AXES_IN_DEV * MOT_MAX_DEV - 1) As Boolean

    Private m_cardIds(15) As Integer
    Private m_cardCount As Integer
    Private m_curCardNo As Integer

#Region "窗口初始化与关闭"
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Dim i, rc, axis As Integer
        Dim Ids As List(Of Integer) = New List(Of Integer)

        For i = 0 To MOT_DIO_IN_DEV - 1
            m_din(i) = 1
        Next

        For i = 0 To MOT_AXES_IN_DEV * MOT_MAX_DEV - 1
            m_AxisJogPos1(i) = 0
            m_AxisJogPos2(i) = 20000
        Next

        dgrdAxisStatus.Rows.Add()
        dgrdAxisStatus.Rows.Add()
        dgrdAxisStatus.Rows.Add()
        dgrdAxisStatus.Rows.Add()
        dgrdAxisStatus.Rows.Add()
        dgrdAxisStatus.Rows.Add()
        dgrdAxisStatus.Rows.Add()
        dgrdAxisStatus.Rows.Add()

        dgrdAxisStatus.Rows(0).HeaderCell.Value = "X"
        dgrdAxisStatus.Rows(1).HeaderCell.Value = "Y"
        dgrdAxisStatus.Rows(2).HeaderCell.Value = "Z"
        dgrdAxisStatus.Rows(3).HeaderCell.Value = "A"
        dgrdAxisStatus.Rows(4).HeaderCell.Value = "B"
        dgrdAxisStatus.Rows(5).HeaderCell.Value = "C"
        dgrdAxisStatus.Rows(6).HeaderCell.Value = "U"
        dgrdAxisStatus.Rows(7).HeaderCell.Value = "V"
        dgrdAxisStatus.RowHeadersWidth() = 65
        dgrdAxisStatus.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
        For i = 0 To dgrdAxisStatus.ColumnCount() - 1
            dgrdAxisStatus.Columns(i).SortMode = DataGridViewColumnSortMode.NotSortable
        Next

        '位置比较器页面设置
        cbCmpRefSource.SelectedIndex = 0

        Timer1.Enabled = True

        NcConfig.Load()

        rc = p9018_open(m_cardCount, m_cardIds)
        If rc <> 0 Then
            MsgBox("控制卡初始化失败！(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
            Application.Exit()
        End If

        If m_cardCount < 1 Then
            MsgBox("找不到控制卡！", MsgBoxStyle.Information, "错误提示")
            Application.Exit()
        Else
            '检查卡号是否有重复
            Ids.Clear()
            For i = 0 To m_cardCount - 1
                If Ids.Contains(m_cardIds(i)) Then
                    MsgBox("检测到重复卡号，请正确设置卡号拨码开关！", MsgBoxStyle.Information, "错误提示")
                    p9018_close()
                    Application.Exit()
                Else
                    Ids.Add(m_cardIds(i))
                End If
            Next

            For i = 0 To m_cardCount - 1

                'config axis according to ncconfig
                For axis = m_cardIds(i) * MOT_AXES_IN_DEV + 0 To m_cardIds(i) * MOT_AXES_IN_DEV + MOT_AXES_IN_DEV - 1
                    ConfigAxis(axis)

                    '为伺服使能设置初始值
                    p9018_axis_set_servo_on(axis, NcConfig.AxisServoON(axis))
                Next

                '将查找到的卡号添加到菜单项中
                ToolStripMenuItemDevSel.DropDownItems.Add(m_cardIds(i).ToString(), Nothing, AddressOf ToolStripMenuItemDevSel_Click).Name = m_cardIds(i).ToString()
            Next
            'CbCardNo.SelectedIndex = 0
            m_curCardNo = m_cardIds(0)
            Me.Text = "PCI-9018#" + m_curCardNo.ToString()

            'update content of axis/dio/cfg page
            AxisUpdateDlg()
            CfgUpateDlg()
            UpdateDio()
        End If

    End Sub

    Private Sub Form1_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        p9018_close()
    End Sub
#End Region

#Region "定时器事件处理"
    ''' <summary>
    ''' 定时器函数处理界面内容刷新
    ''' </summary>
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        Dim axis, row, rc As Integer
        Dim Ver(0 To 3) As UInteger
        Dim info(0 To 7) As UInteger
        Dim i As Integer
        Dim dout As UInteger
        Dim cmdPos, actPos, inpos, posHardLimit, negHardLimit, homeSwitch, fault, abort As Integer
        Dim stat As UInteger
        Dim cmdVel As Double
        Dim axisStat As MOT_AXIS_STAT
        Dim cmpStat As MOT_CMP_STAT
        Dim err As Integer

        For row = 0 To MOT_AXES_IN_DEV - 1
            axis = m_curCardNo * MOT_AXES_IN_DEV + row


            rc = p9018_axis_get_status(axis, axisStat)
            If rc <> 0 Then
                Return
            End If
            cmdPos = CInt(axisStat.cmdPos)
            cmdVel = axisStat.cmdVel
            actPos = CInt(axisStat.actPos)
            inpos = axisStat.inpos
            posHardLimit = axisStat.posHardLimit
            negHardLimit = axisStat.negHardLimit
            homeSwitch = axisStat.homeSwitch
            fault = axisStat.fault
            abort = axisStat.abort
            err = axisStat.err

            '此处更新位置比较器使用的反馈位置
            If NcConfig.CmpEnable(m_curCardNo) <> 0 And (axis Mod MOT_AXES_IN_DEV = NcConfig.CmpRefSource(m_curCardNo)) Then
                TxtCmpActPos.Text = actPos.ToString()
            End If

            dgrdAxisStatus.Rows(row).Cells(0).Value() = cmdPos.ToString()
            dgrdAxisStatus.Rows(row).Cells(1).Value() = actPos.ToString()
            dgrdAxisStatus.Rows(row).Cells(2).Value() = cmdVel.ToString("f1")
            dgrdAxisStatus.Rows(row).Cells(3).Value() = If(inpos <> 0, "Y", "N")
            dgrdAxisStatus.Rows(row).Cells(4).Value() = If(posHardLimit <> 0, "Y", "N")
            dgrdAxisStatus.Rows(row).Cells(4).Style.BackColor = If(posHardLimit <> 0, Color.Red, Color.White)
            dgrdAxisStatus.Rows(row).Cells(5).Value() = If(negHardLimit <> 0, "Y", "N")
            dgrdAxisStatus.Rows(row).Cells(5).Style.BackColor = If(negHardLimit <> 0, Color.Red, Color.White)
            dgrdAxisStatus.Rows(row).Cells(6).Value() = If(homeSwitch <> 0, "Y", "N")
            dgrdAxisStatus.Rows(row).Cells(7).Value() = If(fault <> 0, "Y", "N")
            dgrdAxisStatus.Rows(row).Cells(7).Style.BackColor = If(fault <> 0, Color.Red, Color.White)


            dgrdAxisStatus.Rows(row).Cells(8).Value() = If(abort <> 0, "Y", "N")
            dgrdAxisStatus.Rows(row).Cells(8).Style.BackColor = If(abort <> 0, Color.Red, Color.White)
            'dgrdAxisStatus.Rows(row).Cells(8).Value() = If(err <> 0, "Y", "N")
            'dgrdAxisStatus.Rows(row).Cells(8).Style.BackColor = If(err <> 0, Color.Red, Color.White)
            dgrdAxisStatus.Rows(row).Cells(9).Value() = If(NcConfig.AxisServoON(axis) <> 0, "Y", "N")

            '处理控制轴往复运动
            If m_AxisRepeatMove(axis) And inpos <> 0 Then
                If Math.Abs(cmdPos - m_AxisJogPos1(axis)) < 1 Then

                    p9018_axis_set_profile(axis, NcConfig.AxisMinVel(axis), NcConfig.AxisMaxVel(axis), NcConfig.AxisAccel(axis), NcConfig.AxisDecel(axis), NcConfig.AxisAccel(axis) * 1000000.0)
                    p9018_axis_abs_move(axis, m_AxisJogPos2(axis))

                ElseIf Math.Abs(cmdPos - m_AxisJogPos2(axis)) < 1 Then
                    p9018_axis_set_profile(axis, NcConfig.AxisMinVel(axis), NcConfig.AxisMaxVel(axis), NcConfig.AxisAccel(axis), NcConfig.AxisDecel(axis), NcConfig.AxisAccel(axis) * 1000000.0)
                    p9018_axis_abs_move(axis, m_AxisJogPos1(axis))
                End If
            End If
        Next

        'update DI/DO
        stat = 0
        p9018_motion_get_din_ex(m_curCardNo, stat)  '该接口读出的DIN状态，1表示导通，0表示截止
        For i = 0 To MOT_DIO_IN_DEV - 1
            m_din(i) = If((stat And (&H1 << i)) <> 0, 1, 0)
        Next
        UpdateDio()

        dout = 0
        For i = 0 To MOT_DIO_IN_DEV - 1
            If m_dout(i) <> 0 Then
                dout = dout + (Convert.ToUInt32(&H1) << i)
            End If
        Next
        p9018_motion_set_dout_ex(m_curCardNo, dout) '该接口设置的DOUT状态，1表示导通，0表示截止

        'position comparator
        rc = p9018_cmp_get_status(m_curCardNo, 0, cmpStat)
        If rc = 0 Then
            TxtCmpMatchCount.Text = cmpStat.matchCount.ToString()
            '在更新控制轴状态时更新此状态值
            'TxtCmpActPos.Text = actPos.ToString()
            TxtCmpCurrentRef.Text = cmpStat.curRef.ToString()
            TxtCmpQueueEmpty.Text = If(cmpStat.queEmpty <> 0, "Y", "N")
            TxtCmpQueueFull.Text = If(cmpStat.queFull <> 0, "Y", "N")
            TxtCmpRefCount.Text = cmpStat.queItem.ToString()
        End If

        'show version number
        p9018_motion_get_info(m_curCardNo, info, 8)

        LblAppVer.Text = g_AppVersionStr
        lblApiVer.Text = ((info(6) >> 24) And &HFF).ToString() & "." & ((info(6) >> 16) And &HFF).ToString() & "." & _
                            ((info(6) >> 8) And &HFF).ToString() & "." & ((info(6) >> 0) And &HFF).ToString() & _
                            "(0x" & info(6).ToString("x8") & ")"
        LblDriverVer.Text = ((info(5) >> 24) And &HFF).ToString() & "." & ((info(5) >> 16) And &HFF).ToString() & "." & _
                            ((info(5) >> 8) And &HFF).ToString() & "." & ((info(5) >> 0) And &HFF).ToString() & _
                            "(0x" & info(5).ToString("x8") & ")"
        LblLogicVer.Text = ((info(4) >> 8) And &HFF).ToString() & "." & (info(4) And &HFF).ToString() & "(0x" & info(4).ToString("x4") & ")"
    End Sub

    Sub UpdateDio()
        Dim ChkDIs() As CheckBox = {ChkDI0, ChkDI1, ChkDI2, ChkDI3, ChkDI4, ChkDI5, ChkDI6, ChkDI7, _
        ChkDI8, ChkDI9, ChkDI10, ChkDI11, ChkDI12, ChkDI13, ChkDI14, ChkDI15}

        Dim ChkDOs() As CheckBox = {ChkDO0, ChkDO1, ChkDO2, ChkDO3, ChkDO4, ChkDO5, ChkDO6, ChkDO7, _
        ChkDO8, ChkDO9, ChkDO10, ChkDO11, ChkDO12, ChkDO13, ChkDO14, ChkDO15}

        Dim i As Integer
        For i = 0 To MOT_DIO_IN_DEV - 1
            ChkDIs(i).Checked = If((m_din(i) <> 0), True, False)
            m_dout(i) = If(ChkDOs(i).Checked, 1, 0)
        Next
    End Sub
#End Region

#Region "控制轴运动页面处理"
    ''' <summary>
    ''' 控制轴运动页面获取选中的控制轴
    ''' </summary>
    Private Function GetAxisSel() As Integer
        If RadAxisSel0.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 0
        ElseIf RadAxisSel1.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 1
        ElseIf RadAxisSel2.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 2
        ElseIf RadAxisSel3.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 3
        ElseIf RadAxisSel4.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 4
        ElseIf RadAxisSel5.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 5
        ElseIf RadAxisSel6.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 6
        Else
            Return m_curCardNo * MOT_AXES_IN_DEV + 7
        End If
    End Function
    ''' <summary>
    ''' 控制轴运动页面运行按钮
    ''' </summary>
    Private Sub BtnAxisRun_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAxisRun.Click
        Dim minVel, maxVel, accel, decel As Double
        Dim axis, rc, pos1, pos2 As Integer

        pos1 = CInt(TxtAxisPos1.Text)
        pos2 = CInt(TxtAxisPos2.Text)
        minVel = CDbl(TxtAxisMinVel.Text)
        maxVel = CDbl(TxtAxisMaxVel.Text)
        accel = CDbl(TxtAxisAccel.Text)
        decel = CDbl(TxtAxisDecel.Text)

        axis = GetAxisSel()

        'donot change minvel/maxvel/accel/decel while operating cmdPos and fbPos
        If Not RadAxisSetCmdPos.Checked And Not RadAxisSetFbPos.Checked Then
            'donnot support s-curve
            rc = p9018_axis_set_profile(axis, minVel, maxVel, accel, decel, accel * 1000000.0)
        End If
        NcConfig.AxisMinVel(axis) = minVel
        NcConfig.AxisMaxVel(axis) = maxVel
        NcConfig.AxisAccel(axis) = accel
        NcConfig.AxisDecel(axis) = decel

        If RadAxisAbsJog.Checked Then
            rc = p9018_axis_abs_move(axis, pos1)

        ElseIf RadAxisIncrJog.Checked Then
            rc = p9018_axis_incr_move(axis, pos1)

        ElseIf RadAxisJogPos.Checked Then
            rc = p9018_axis_cont_move(axis, Math.Abs(maxVel))

        ElseIf RadAxisJogNeg.Checked Then
            rc = p9018_axis_cont_move(axis, -1.0 * Math.Abs(maxVel))

        ElseIf RadAxisHomePos.Checked Then
            rc = p9018_axis_home(axis, Math.Abs(maxVel), NcConfig.AxisHomeMode(axis))


        ElseIf RadAxisHomeNeg.Checked Then
            rc = p9018_axis_home(axis, -1 * Math.Abs(maxVel), NcConfig.AxisHomeMode(axis))

        ElseIf RadAxisRepeatMove.Checked Then
            If Not m_AxisRepeatMove(axis) Then
                rc = p9018_axis_abs_move(axis, pos1)
                m_AxisJogPos1(axis) = pos1
                m_AxisJogPos2(axis) = pos2
                m_AxisRepeatMove(axis) = True

            End If
        ElseIf RadAxisSetCmdPos.Checked Then
            rc = p9018_axis_set_cmd_pos(axis, pos1)

        ElseIf RadAxisSetFbPos.Checked Then
            rc = p9018_axis_set_act_pos(axis, pos1)

        End If

        If rc <> 0 Then
            MsgBox("发送命令失败(" & rc.ToString() & "-" & GetErrorString(rc) & ")", MsgBoxStyle.Information, "错误提示")
        End If
    End Sub

    Private Sub BtnAxisAbort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAxisAbort.Click
        Dim rc, axis As Integer

        axis = GetAxisSel()
        If m_AxisRepeatMove(axis) Then
            m_AxisRepeatMove(axis) = False
        End If

        rc = p9018_axis_abort(axis)
        If rc <> 0 Then
            MsgBox("发送命令失败(" & rc.ToString() & "-" & GetErrorString(rc) & ")", MsgBoxStyle.Information, "错误提示")
        End If
    End Sub

    ''' <summary>
    ''' 切换控制轴时更新界面显示
    ''' </summary>
    Private Sub RadAxisSel_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadAxisSel0.CheckedChanged, RadAxisSel1.CheckedChanged, RadAxisSel2.CheckedChanged, RadAxisSel3.CheckedChanged, _
        RadAxisSel4.CheckedChanged, RadAxisSel5.CheckedChanged, RadAxisSel6.CheckedChanged, RadAxisSel7.CheckedChanged
        AxisUpdateDlg()
    End Sub
    ''' <summary>
    ''' 控制轴运动页面自动更新参数，切换卡号/控制轴时调用
    ''' </summary>
    Private Sub AxisUpdateDlg()
        Dim axis, index As Integer

        axis = GetAxisSel()
        If axis < 0 Then
            Return
        End If

        TxtAxisPos1.Text = m_AxisJogPos1(axis).ToString()
        TxtAxisPos2.Text = m_AxisJogPos2(axis).ToString()
        TxtAxisMinVel.Text = NcConfig.AxisMinVel(axis).ToString("f1")
        TxtAxisMaxVel.Text = NcConfig.AxisMaxVel(axis).ToString("f1")
        TxtAxisAccel.Text = NcConfig.AxisAccel(axis).ToString("f1")
        TxtAxisDecel.Text = NcConfig.AxisDecel(axis).ToString("f1")

        '控制轴选择控件里面显示字母轴号和数字轴号
        index = m_curCardNo * MOT_AXES_IN_DEV + 0
        RadAxisSel0.Text = "X/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 1
        RadAxisSel1.Text = "Y/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 2
        RadAxisSel2.Text = "Z/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 3
        RadAxisSel3.Text = "A/" + index.ToString()

        index = m_curCardNo * MOT_AXES_IN_DEV + 4
        RadAxisSel4.Text = "B/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 5
        RadAxisSel5.Text = "C/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 6
        RadAxisSel6.Text = "U/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 7
        RadAxisSel7.Text = "V/" + index.ToString()

    End Sub
    ''' <summary>
    ''' 控制轴指令选择改变时，调整界面控件使能状态
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub RadAxisCmdType_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles _
        RadAxisAbsJog.CheckedChanged, RadAxisIncrJog.CheckedChanged, RadAxisJogPos.CheckedChanged, RadAxisJogNeg.CheckedChanged, _
        RadAxisHomePos.CheckedChanged, RadAxisHomeNeg.CheckedChanged, RadAxisRepeatMove.CheckedChanged, _
        RadAxisSetCmdPos.CheckedChanged, RadAxisSetFbPos.CheckedChanged

        If RadAxisRepeatMove.Checked Then
            TxtAxisPos2.Enabled = True
            LblAxisPos2.Enabled = True
        Else
            TxtAxisPos2.Enabled = False
            LblAxisPos2.Enabled = False
        End If

        '连续运动时，禁止不必要的控件
        If RadAxisJogPos.Checked Or RadAxisJogNeg.Checked Or RadAxisHomePos.Checked Or RadAxisHomeNeg.Checked Then
            LblAxisPos1.Enabled = False
            TxtAxisPos1.Enabled = False
        Else
            LblAxisPos1.Enabled = True
            TxtAxisPos1.Enabled = True
        End If

        '设置指令位置或反馈位置，禁止不必要的控件
        If RadAxisSetCmdPos.Checked Or RadAxisSetFbPos.Checked Then
            LblAxisMinVel.Enabled = False
            TxtAxisMinVel.Enabled = False
            LblAxisMaxVel.Enabled = False
            TxtAxisMaxVel.Enabled = False
            LblAxisAccel.Enabled = False
            TxtAxisAccel.Enabled = False
            LblAxisDecel.Enabled = False
            TxtAxisDecel.Enabled = False
            'LblAxisJerk.Enabled = False
            'TxtAxisJerk.Enabled = False
        Else
            LblAxisMinVel.Enabled = True
            TxtAxisMinVel.Enabled = True
            LblAxisMaxVel.Enabled = True
            TxtAxisMaxVel.Enabled = True
            LblAxisAccel.Enabled = True
            TxtAxisAccel.Enabled = True
            LblAxisDecel.Enabled = True
            TxtAxisDecel.Enabled = True
            'LblAxisJerk.Enabled = True
            'TxtAxisJerk.Enabled = True
        End If
    End Sub

    Private Sub BtnAxisServoONToggle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnAxisServoONToggle.Click
        Dim axis As Integer

        axis = GetAxisSel()
        If axis < 0 Then
            Return
        End If

        NcConfig.AxisServoON(axis) = If(NcConfig.AxisServoON(axis) <> 0, 0, 1)
        p9018_axis_set_servo_on(axis, NcConfig.AxisServoON(axis))
    End Sub
    ''' <summary>
    ''' 将错误码转换为字符串
    ''' </summary>
    Private Function GetErrorString(ByVal err As Integer) As String
        If err = 1 Then
            Return "无效参数"
        ElseIf err = 2 Then
            Return "分配系统资源失败"
        ElseIf err = 7 Then
            Return "无法访问驱动程序"
        ElseIf err = 12 Then
            Return "设备已经打开"

        ElseIf err = 201 Then
            Return "无法识别的运动错误"
        ElseIf err = 202 Then
            Return "控制轴忙"
        ElseIf err = 205 Then
            Return "控制轴限位或告警有效"
        Else
            Return ""
        End If
    End Function

#End Region

#Region "配置界面处理"
    ''' <summary>
    ''' 获取配置标签页选中的控制轴
    ''' </summary>
    Private Function CfgGetAxisSel() As Integer
        If RadCfgAxisSel0.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 0
        ElseIf RadCfgAxisSel1.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 1
        ElseIf RadCfgAxisSel2.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 2
        ElseIf RadCfgAxisSel3.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 3
        ElseIf RadCfgAxisSel4.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 4
        ElseIf RadCfgAxisSel5.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 5
        ElseIf RadCfgAxisSel6.Checked Then
            Return m_curCardNo * MOT_AXES_IN_DEV + 6
        Else
            Return m_curCardNo * MOT_AXES_IN_DEV + 7
        End If
    End Function
    ''' <summary>
    ''' 参数设置标签页更新界面显示
    ''' </summary>
    Private Sub CfgUpateDlg()
        Dim axis, index As Integer

        axis = CfgGetAxisSel()

        RadAxisOutputPulDir.Checked = If(NcConfig.AxisOutputMode(axis) = 0, True, False)
        RadAxisOutputCwCcw.Checked = If(NcConfig.AxisOutputMode(axis) = 1, True, False)

        RadOutputInvDisable.Checked = If(NcConfig.AxisOutputDirInv(axis) = 0, True, False)
        RadOutputInvEnable.Checked = If(NcConfig.AxisOutputDirInv(axis) = 1, True, False)

        RadAxisInputPulDir.Checked = If(NcConfig.AxisInputMode(axis) = 0, True, False)
        RadAxisInputQuadrature.Checked = If(NcConfig.AxisInputMode(axis) = 1, True, False)

        RadInputInvDisable.Checked = If(NcConfig.AxisInputDirInv(axis) = 0, True, False)
        RadInputInvEnable.Checked = If(NcConfig.AxisInputDirInv(axis) = 1, True, False)

        RadPHLNormalOpen.Checked = If(NcConfig.AxisPelType(axis) = 0, True, False)
        RadPHLNormalClose.Checked = If(NcConfig.AxisPelType(axis) = 1, True, False)

        RadPHLEnable.Checked = If(NcConfig.AxisPelEnable(axis) = 1, True, False)
        RadPHLDisable.Checked = If(NcConfig.AxisPelEnable(axis) = 0, True, False)
        'Setting NHL type is not supported

        RadNHLEnable.Checked = If(NcConfig.AxisNelEnable(axis) = 1, True, False)
        RadNHLDisable.Checked = If(NcConfig.AxisNelEnable(axis) = 0, True, False)

        RadFaultDisable.Checked = If(NcConfig.AxisFaultEnable(axis) = 0, True, False)
        RadFaultEnable.Checked = If(NcConfig.AxisFaultEnable(axis) = 1, True, False)
        RadFaultNormalOpen.Checked = If(NcConfig.AxisFaultType(axis) = 0, True, False)
        RadFaultNormalClose.Checked = If(NcConfig.AxisFaultType(axis) = 1, True, False)

        RadHomeOnly.Checked = If(NcConfig.AxisHomeMode(axis) = 0, True, False)
        RadHomeAndIndex.Checked = If(NcConfig.AxisHomeMode(axis) = 1, True, False)

        RadHomeSwNormalOpen.Checked = If(NcConfig.AxisHomeSwType(axis) = 0, True, False)
        RadHomeSwNormalClose.Checked = If(NcConfig.AxisHomeSwType(axis) = 1, True, False)

        RadIndexRisingEdge.Checked = If(NcConfig.AxisIndexPolarity(axis) = 1, True, False)
        RadIndexFallingEdge.Checked = If(NcConfig.AxisIndexPolarity(axis) = 0, True, False)

        'RadAxisProfileT.Checked = if(NcConfig.AxisProfileType(axis) = 0, True, False)
        'RadAxisProfileS.Checked = if(NcConfig.AxisProfileType(axis) = 1, True, False)
        'TxtAxisJerkPercent.Text = NcConfig.AxisProfileJerkPercent(axis).ToString("f1")

        'ABORT为共享信号，通过X轴接口配置ABORT
        If axis Mod MOT_AXES_IN_DEV = 0 Then
            RadAbortNormalClose.Checked = If(NcConfig.AbortType(axis \ MOT_AXES_IN_DEV) <> 0, True, False)
            RadAbortEnable.Checked = If(NcConfig.AbortEnable(axis \ MOT_AXES_IN_DEV) <> 0, True, False)
        End If

        '更新比较器显示状态
        RadCmpEnable.Checked = If(NcConfig.CmpEnable(m_curCardNo) <> 0, True, False)
        RadCmpDisable.Checked = If(NcConfig.CmpEnable(m_curCardNo) = 0, True, False)

        TxtCmpOutLength.Text = NcConfig.CmpOutLength(m_curCardNo).ToString()

        RadCmpOutLevelHigh.Checked = If(NcConfig.CmpOutActiveLevel(m_curCardNo) <> 0, True, False)
        RadCmpOutLevelLow.Checked = If(NcConfig.CmpOutActiveLevel(m_curCardNo) = 0, True, False)
        'cbCmpRefSource.SelectedIndex = NcConfig.CmpRefSource(m_curCardNo)

        '控制轴选择控件里面显示字母轴号和数字轴号
        index = m_curCardNo * MOT_AXES_IN_DEV + 0
        RadCfgAxisSel0.Text = "X/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 1
        RadCfgAxisSel1.Text = "Y/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 2
        RadCfgAxisSel2.Text = "Z/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 3
        RadCfgAxisSel3.Text = "A/" + index.ToString()

        index = m_curCardNo * MOT_AXES_IN_DEV + 4
        RadCfgAxisSel4.Text = "B/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 5
        RadCfgAxisSel5.Text = "C/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 6
        RadCfgAxisSel6.Text = "U/" + index.ToString()
        index = m_curCardNo * MOT_AXES_IN_DEV + 7
        RadCfgAxisSel7.Text = "V/" + index.ToString()

        '选中X轴时才允许配置ABORT
        GroupBoxCfgAbort.Enabled = If(RadCfgAxisSel0.Checked, True, False)
    End Sub
    ''' <summary>
    ''' 更新DI/DO界面，卡号更改后调用
    ''' </summary>
    Private Sub DioUpdateDlg()
        Dim ChkDOs() As CheckBox = {ChkDO0, ChkDO1, ChkDO2, ChkDO3, ChkDO4, ChkDO5, ChkDO6, ChkDO7, _
                                    ChkDO8, ChkDO9, ChkDO10, ChkDO11, ChkDO12, ChkDO13, ChkDO14, ChkDO15}
        Dim d As UInteger
        Dim i, rc As Integer

        rc = p9018_motion_get_dout_ex(m_curCardNo, d)
        If rc <> 0 Then
            Return
        End If
        'DI/DO state definition: 1 for ON, 0 for OFF
        For i = 0 To MOT_DIO_IN_DEV - 1
            ChkDOs(i).Checked = If((d And (&H1 << i)) <> 0, True, False)
        Next

    End Sub
    ''' <summary>
    ''' 参数设置控制轴改变时，更新显示内容
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub RadCfgAxisSel_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RadCfgAxisSel0.CheckedChanged, RadCfgAxisSel1.CheckedChanged, RadCfgAxisSel2.CheckedChanged, RadCfgAxisSel3.CheckedChanged, _
        RadCfgAxisSel4.CheckedChanged, RadCfgAxisSel5.CheckedChanged, RadCfgAxisSel6.CheckedChanged, RadCfgAxisSel7.CheckedChanged
        CfgUpateDlg()
    End Sub
    ''' <summary>
    ''' 配置标签页保存参数并配置控制轴
    ''' </summary>
    Private Sub BtnCfgApply_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnCfgApply.Click
        Dim axis As Integer

        axis = CfgGetAxisSel()

        If RadAxisOutputPulDir.Checked Then
            NcConfig.AxisOutputMode(axis) = 0
        Else 'RadAxisOutputCwCcw.Checked Then
            NcConfig.AxisOutputMode(axis) = 1
            'Else
            ' NcConfig.AxisOutputMode(axis) = 2
        End If

        NcConfig.AxisOutputDirInv(axis) = If(RadOutputInvEnable.Checked, 1, 0)
        NcConfig.AxisInputMode(axis) = If(RadAxisInputPulDir.Checked, 0, 1)
        NcConfig.AxisInputDirInv(axis) = If(RadInputInvEnable.Checked, 1, 0)

        'PEL
        If RadPHLNormalClose.Checked Then
            NcConfig.AxisPelType(axis) = 1
        Else
            NcConfig.AxisPelType(axis) = 0
        End If

        NcConfig.AxisPelEnable(axis) = If(RadPHLEnable.Checked, 1, 0)
        'If RadNHLNormalClose.Checked Then
        '    NcConfig.AxisNelType(axis) = 1
        'Else
        '    NcConfig.AxisNelType(axis) = 0
        'End If
        NcConfig.AxisNelEnable(axis) = If(RadNHLEnable.Checked, 1, 0)

        'FAULT
        If RadFaultEnable.Checked Then
            NcConfig.AxisFaultEnable(axis) = 1
        Else
            NcConfig.AxisFaultEnable(axis) = 0
        End If

        If RadFaultNormalClose.Checked Then
            NcConfig.AxisFaultType(axis) = 1
        Else
            NcConfig.AxisFaultType(axis) = 0
        End If

        'home, index
        If RadHomeSwNormalClose.Checked Then
            NcConfig.AxisHomeSwType(axis) = 1
        Else
            NcConfig.AxisHomeSwType(axis) = 0
        End If

        If RadHomeAndIndex.Checked Then
            NcConfig.AxisHomeMode(axis) = 1
        Else
            NcConfig.AxisHomeMode(axis) = 0
        End If

        If RadIndexRisingEdge.Checked Then
            NcConfig.AxisIndexPolarity(axis) = 1
        Else
            NcConfig.AxisIndexPolarity(axis) = 0
        End If

        'ABORT为共享信号，通过X轴接口配置ABORT
        If axis Mod MOT_AXES_IN_DEV = 0 Then
            NcConfig.AbortType(axis \ MOT_AXES_IN_DEV) = If(RadAbortNormalClose.Checked, 1, 0)
            NcConfig.AbortEnable(axis \ MOT_AXES_IN_DEV) = If(RadAbortEnable.Checked, 1, 0)
        End If

        NcConfig.Save()
        ConfigAxis(axis)
    End Sub
    ''' <summary>
    ''' 配置控制轴参数
    ''' </summary>
    Sub ConfigAxis(ByVal axis As Integer)
        Dim rc As Integer

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_OutputMode, NcConfig.AxisOutputMode(axis))
        If rc <> 0 Then
            MsgBox("发送PN_OutputMode命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_InvertOutputDirection, NcConfig.AxisOutputDirInv(axis))
        If rc <> 0 Then
            MsgBox("发送PN_InvertOutputDirection命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_InputMode, NcConfig.AxisInputMode(axis))
        If rc <> 0 Then
            MsgBox("发送PN_InputMode命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_InvertInputDirection, NcConfig.AxisInputDirInv(axis))
        If rc <> 0 Then
            MsgBox("发送PN_InvertInputDirection命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_PosLimitSwitchType, NcConfig.AxisPelType(axis))
        If rc <> 0 Then
            MsgBox("发送PN_PosLimitSwitchType命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_PosLimitSwitchEnable, NcConfig.AxisPelEnable(axis))
        If rc <> 0 Then
            MsgBox("发送PN_PosLimitSwitchEnable命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        'rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_NegLimitSwitchType, NcConfig.AxisNelType(axis))       'not supported
        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_NegLimitSwitchEnable, NcConfig.AxisNelEnable(axis))
        If rc <> 0 Then
            MsgBox("发送PN_NegLimitSwitchEnable命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_AmpFaultType, NcConfig.AxisFaultType(axis))
        If rc <> 0 Then
            MsgBox("发送PN_AmpFaultType命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_AmpFaultEnable, NcConfig.AxisFaultEnable(axis))
        If rc <> 0 Then
            MsgBox("发送命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_HomeSwitchType, NcConfig.AxisHomeSwType(axis))
        If rc <> 0 Then
            MsgBox("发送PN_AmpFaultEnable命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_IndexActiveLevel, NcConfig.AxisIndexPolarity(axis))
        If rc <> 0 Then
            MsgBox("发送PN_IndexActiveLevel命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If

        'ABORT为共享信号，通过X轴接口配置ABORT
        If axis Mod MOT_AXES_IN_DEV = 0 Then
            rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_AbortSwitchType, NcConfig.AbortType(axis \ MOT_AXES_IN_DEV))
            If rc <> 0 Then
                MsgBox("发送PN_AbortSwitchType命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
            End If

            rc = p9018_axis_set_parameter(axis, PN_NUMBER.PN_AbortSwitchEnable, NcConfig.AbortEnable(axis \ MOT_AXES_IN_DEV))
            If rc <> 0 Then
                MsgBox("发送PN_AbortSwitchEnable命令失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
            End If
        End If
    End Sub
#End Region
    'Private Sub CbCardNo_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CbCardNo.SelectedIndexChanged
    '    If CbCardNo.SelectedIndex < 0 Then
    '        Return
    '    End If

    '    m_curCardNo = m_cardIds(CbCardNo.SelectedIndex)

    '    DioUpdateDlg()
    '    CfgUpateDlg()
    '    AxisUpdateDlg()
    'End Sub

    Private Sub ToolStripMenuItemAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ToolStripMenuItemAbout.Click
        Dim dlg As New AboutBox1
        dlg.ShowDialog()
    End Sub

    '使用菜单项来选择控制卡卡号
    Private Sub ToolStripMenuItemDevSel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Me.Text = "PCI-9018#" + sender.ToString()
        m_curCardNo = CInt(sender.ToString())

        DioUpdateDlg()
        CfgUpateDlg()
        AxisUpdateDlg()
    End Sub

    Private Sub BtnCmpSetRef_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnCmpSetRef.Click
        Dim sValues() As String
        Dim separators() As String = {" ", vbTab, ",", ";"}
        Dim rc As Integer

        If NcConfig.CmpEnable(m_curCardNo) = 0 Then
            MsgBox("比较器未使能！")
            Return
        End If

        sValues = TxtCmpPosRef.Text.Split(separators, StringSplitOptions.RemoveEmptyEntries)
        For Each item As String In sValues
            rc = p9018_cmp_set_ref(m_curCardNo, 0, CInt(item))
            If rc <> 0 Then
                MsgBox("设置比较器参考值失败!(" + rc.ToString() + ")")
            End If
        Next

    End Sub


    Private Sub BtnCmpSetParam_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BtnCmpSetParam.Click
        Dim cmpEnable, cmpRefSource, cmpOutLen, cmpOutLevel, rc As Integer

        cmpEnable = If(RadCmpEnable.Checked, 1, 0)

        If cbCmpRefSource.SelectedIndex < 0 Then
            MsgBox("位置参考源选择错误", MsgBoxStyle.Information, "错误提示")
            Return
        End If
        cmpRefSource = cbCmpRefSource.SelectedIndex
        cmpOutLen = CInt(TxtCmpOutLength.Text)
        cmpOutLevel = If(RadCmpOutLevelHigh.Checked, 1, 0)

        NcConfig.CmpOutLength(m_curCardNo) = cmpOutLen
        NcConfig.CmpRefSource(m_curCardNo) = cmpRefSource
        NcConfig.CmpOutActiveLevel(m_curCardNo) = cmpOutLevel
        NcConfig.CmpEnable(m_curCardNo) = cmpEnable

        rc = p9018_cmp_enable(m_curCardNo, 0, cmpEnable, cmpOutLevel, cmpRefSource, cmpOutLen)
        If rc <> 0 Then
            MsgBox("设置位置比较器失败(" & rc.ToString() & ")", MsgBoxStyle.Information, "错误提示")
        End If
    End Sub

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

    End Sub
End Class
