﻿Option Strict On
Option Explicit On

Module NcConfig
    Const MAX_AXIS As Integer = 128     'axes supported by API
    Const MAX_CARD As Integer = 16      'controllers supported by API
    Const AXES_IN_DEV As Integer = 8    'axes in one controller

    Public AxisOutputMode(MAX_AXIS - 1) As Integer
    Public AxisOutputDirInv(MAX_AXIS - 1) As Integer
    Public AxisInputMode(MAX_AXIS - 1) As Integer
    Public AxisInputDirInv(MAX_AXIS - 1) As Integer

    Public AxisPelEnable(MAX_AXIS - 1) As Integer
    Public AxisPelType(MAX_AXIS - 1) As Integer
    Public AxisNelEnable(MAX_AXIS - 1) As Integer
    Public AxisNelType(MAX_AXIS - 1) As Integer
    Public AxisFaultEnable(MAX_AXIS - 1) As Integer
    Public AxisFaultType(MAX_AXIS - 1) As Integer

    Public AxisHomeSwType(MAX_AXIS - 1) As Integer
    Public AxisHomeMode(MAX_AXIS - 1) As Integer
    Public AxisIndexPolarity(MAX_AXIS - 1) As Integer   ' 1 for rising edge; 0 for falling edge

    Public AbortType(MAX_CARD - 1) As Integer       'one controller only has one abort
    Public AbortEnable(MAX_CARD - 1) As Integer

    Public AxisMinVel(MAX_AXIS - 1) As Double
    Public AxisMaxVel(MAX_AXIS - 1) As Double

    Public AxisProfileJerk(MAX_AXIS - 1) As Double
    Public AxisAccel(MAX_AXIS - 1) As Double
    Public AxisDecel(MAX_AXIS - 1) As Double
    Public AxisServoON(MAX_AXIS - 1) As Integer 'servo ON

    Public CmpEnable(MAX_CARD - 1) As Integer
    Public CmpRefSource(MAX_CARD - 1) As Integer
    Public CmpOutLength(MAX_CARD - 1) As Integer
    Public CmpOutActiveLevel(MAX_CARD - 1) As Integer

    Private iniFilePath As String

    Private Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Int32, ByVal lpFileName As String) As Int32
    Private Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal lpString As String, ByVal lpFileName As String) As Int32

    Public Function GetINI(ByVal Section As String, ByVal AppName As String, Optional ByVal lpDefault As String = "") As String
        Dim Str As String = ""
        Str = LSet(Str, 256)
        GetPrivateProfileString(Section, AppName, lpDefault, Str, Len(Str), iniFilePath)
        Return Microsoft.VisualBasic.Left(Str, InStr(Str, Chr(0)) - 1)
    End Function

    Public Function SetINI(ByVal Section As String, ByVal AppName As String, ByVal lpString As String) As Boolean
        Try
            WritePrivateProfileString(Section, AppName, lpString, iniFilePath)
        Catch exception1 As Exception
            Return False
        End Try
        Return True
    End Function


    Sub Load()
        Dim axis, i As Integer
        Dim sectionName As String

        '获取配置文件的路径
        iniFilePath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().ManifestModule.FullyQualifiedName) & _
        System.IO.Path.DirectorySeparatorChar.ToString() & "PCI9018.ini"

        For axis = 0 To MAX_AXIS - 1
            sectionName = "AXIS_" & axis.ToString()
            AxisOutputMode(axis) = CInt(GetINI(sectionName, "OUTPUT_MODE", "0"))
            AxisOutputDirInv(axis) = CInt(GetINI(sectionName, "OUTPUT_DIR_INV", "0"))
            AxisInputMode(axis) = CInt(GetINI(sectionName, "INPUT_MODE", "1"))
            AxisInputDirInv(axis) = CInt(GetINI(sectionName, "INPUT_DIR_INV", "0"))

            AxisPelEnable(axis) = CInt(GetINI(sectionName, "POS_HARD_LIMIT_ENABLE", "1"))
            AxisPelType(axis) = CInt(GetINI(sectionName, "POS_HARD_LIMIT_TYPE", "0"))
            AxisNelEnable(axis) = CInt(GetINI(sectionName, "NEG_HARD_LIMIT_ENABLE", "1"))
            'AxisNelType(axis) = CInt(GetINI(sectionName, "NEG_HARD_LIMIT_TYPE", "0"))
            AxisFaultEnable(axis) = CInt(GetINI(sectionName, "AMP_FAULT_ENABLE", "1"))
            AxisFaultType(axis) = CInt(GetINI(sectionName, "AMP_FAULT_TYPE", "0"))

            AxisHomeSwType(axis) = CInt(GetINI(sectionName, "HOME_SWITCH_TYPE", "0"))
            AxisHomeMode(axis) = CInt(GetINI(sectionName, "USE_INDEX", "0"))
            AxisIndexPolarity(axis) = CInt(GetINI(sectionName, "INDEX_POLARITY", "1"))
            'AxisProfileType(axis) = CInt(GetINI(sectionName, "VEL_PROFILE", "0"))
            'AxisProfileJerkPercent(axis) = CDbl(GetINI(sectionName, "JERK_PERCENT", "0.3"))
            AxisProfileJerk(axis) = CDbl(GetINI(sectionName, "JERK", "500000000.0"))

            AxisMinVel(axis) = CDbl(GetINI(sectionName, "MIN_VEL", "1000.0"))
            AxisMaxVel(axis) = CDbl(GetINI(sectionName, "MAX_VEL", "100000.0"))
            AxisAccel(axis) = CDbl(GetINI(sectionName, "ACCELERATION", "500000.0"))
            AxisDecel(axis) = CDbl(GetINI(sectionName, "DECELERATION", "500000.0"))

            AxisServoON(axis) = 0

            If axis Mod AXES_IN_DEV = 0 Then
                AbortEnable(axis \ AXES_IN_DEV) = CInt(GetINI(sectionName, "ABORT_ENABLE", "1"))
                AbortType(axis \ AXES_IN_DEV) = CInt(GetINI(sectionName, "ABORT_TYPE", "0"))
            End If
        Next

        For i = 0 To MAX_CARD - 1
            CmpEnable(i) = 0
            CmpOutActiveLevel(i) = 0
            CmpOutLength(i) = 100000
            CmpRefSource(i) = 0
        Next
    End Sub

    Sub Save()
        Dim axis As Integer
        Dim sectionName As String

        For axis = 0 To MAX_AXIS - 1
            sectionName = "AXIS_" & axis.ToString()

            SetINI(sectionName, "OUTPUT_MODE", AxisOutputMode(axis).ToString())
            SetINI(sectionName, "OUTPUT_DIR_INV", AxisOutputDirInv(axis).ToString())
            SetINI(sectionName, "INPUT_MODE", AxisInputMode(axis).ToString())
            SetINI(sectionName, "INPUT_DIR_INV", AxisInputDirInv(axis).ToString())

            SetINI(sectionName, "POS_HARD_LIMIT_ENABLE", AxisPelEnable(axis).ToString())
            SetINI(sectionName, "POS_HARD_LIMIT_TYPE", AxisPelType(axis).ToString())
            SetINI(sectionName, "NEG_HARD_LIMIT_ENABLE", AxisNelEnable(axis).ToString())
            'SetINI(sectionName, "NEG_HARD_LIMIT_TYPE", AxisNelType(axis).ToString())
            SetINI(sectionName, "AMP_FAULT_ENABLE", AxisFaultEnable(axis).ToString())
            SetINI(sectionName, "AMP_FAULT_TYPE", AxisFaultType(axis).ToString())

            SetINI(sectionName, "HOME_SWITCH_TYPE", AxisHomeSwType(axis).ToString())
            SetINI(sectionName, "USE_INDEX", AxisHomeMode(axis).ToString())
            SetINI(sectionName, "INDEX_POLARITY", AxisIndexPolarity(axis).ToString())

            'SetINI(sectionName, "VEL_PROFILE", AxisProfileType(axis).ToString())
            'SetINI(sectionName, "JERK_PERCENT", AxisProfileJerkPercent(axis).ToString("f1"))
            SetINI(sectionName, "JERK", AxisProfileJerk(axis).ToString("f1"))

            SetINI(sectionName, "MIN_VEL", AxisMinVel(axis).ToString("f1"))
            SetINI(sectionName, "MAX_VEL", AxisMaxVel(axis).ToString("f1"))
            SetINI(sectionName, "ACCELERATION", AxisAccel(axis).ToString("f1"))
            SetINI(sectionName, "DECELERATION", AxisDecel(axis).ToString("f1"))

            If axis Mod AXES_IN_DEV = 0 Then
                SetINI(sectionName, "ABORT_ENABLE", AbortEnable(axis \ AXES_IN_DEV).ToString())
                SetINI(sectionName, "ABORT_TYPE", AbortType(axis \ AXES_IN_DEV).ToString())
            End If
        Next
    End Sub
End Module
