//
//  source file of motion controller demo
//
#include "stdafx.h"
#include <windows.h>    //Sleep, UINT, QueryPerformanceCounter, QueryPerformanceFrequency
#include <stdio.h>
#include <stdarg.h>
#include "PCI9018.h"

#define AXIS_HOMING_TIMEOUT  (30000) //ԭʱ䣬λms

int capture_home(int axis, double vel, int use_index);
int wait_axis(int axis, int timeout_ms);
int dump_axis(int axis);
double etime();
int myPrint(const char *_fmt, ...);

/*! \fn int _tmain(int argc, _TCHAR* argv[])
 *  \brief entry point
 */
int _tmain(int argc, _TCHAR* argv[])
{
    int count = 0, board_id[16], rc = 0;
    int axis, card_no, pos = 0;
    int err = 0;

    rc = p9018_open(&count, board_id);
    if(rc)
    {
        myPrint("Open fails\n" );
        return -1;
    }

    if(count < 1)
    {
        myPrint("Find no card\n" );
        p9018_close();
        return -1;
    }

    card_no = board_id[0];
    axis = card_no*8 + 0;	//ƿϵһ
    pos = 10000;

    rc = p9018_axis_set_profile(axis, 1000.0, 100000.0, 1000000.0, 1000000.0, 1000000000.0);
    if(rc)
        myPrint("axis_set_profile[%d] fails, rc: %d\n", axis, rc);

    //Ȳԭ㣬ɹִеλ˶
    rc = capture_home(axis, -5000.0, 0);
    if(rc){
        myPrint("capture_home[%d] fails(%d)\n", axis, rc);
    }
    else{
        rc = p9018_axis_abs_move(axis, pos);
        if(rc)
            myPrint("axis_abs_move[%d] fails, rc: %d\n", axis, rc);

        rc = wait_axis(axis, 0);
        if(rc){
            myPrint("wait_axis[%d] fails(%d)\n", axis, rc);
        }

    }
    dump_axis(axis);

    //close all card
    p9018_close();
    return 0;
}

/*! \fn int wait_axis(int axis)
 *  \brief ȴֹͣ˶ֹͣ쳣ֹͣʱ
 *
 *  \param axis - 
 *  \param timeout_ms - ʱ䣬λ:msǼ鳬ʱСڵʱ鳬ʱ
 *
 *  \return 0-ʾֹͣ˶йδ쳣1-쳣ֹͣ˶2-ʱ
 */
int wait_axis(int axis, int timeout_ms)
{
    int rc = 0;
    double t;
    MOT_AXIS_STAT axis_status;

    axis_status.error = 0;
    t = etime();
    while(true)
    {
        rc = p9018_axis_get_status(axis, &axis_status);
        if(!rc && axis_status.inpos)
            break;

        Sleep(1);   //ͷϵͳԴ

        if((timeout_ms > 0) && (etime() > (t + timeout_ms * 0.001)))
        {
            //ʱ
            return 2;
        }
    }
    //ֹͣʱδ쳣
    if(!axis_status.error)
        return 0;

    //쳣ֹͣ
    return 1;
}

/*! \fn int capture_home(int axis, double vel, int use_index)
 *  \brief βԭ㷶
 *
 *  ԭҿɼٶȿ;ȸߵŵ
 *
 *  \param axis - 
 *  \param vel - ԭٶȣž
 *  \param use_index - ǷIndex
 *
 *  \return ʾгɹʾд
 */
int capture_home(int axis, double vel, int use_index)
{
    int rc = 0;
    MOT_AXIS_STAT axis_status;
    //ԭǰԭźЧʱ˵ľ
    //ʹѡֵʹÿܹ뿪ԭ
    int step_dist = 2000;

    //һβԭ㣬ԭźѾЧһξ
    rc = p9018_axis_get_status(axis, &axis_status);
    if(!rc){
        if(axis_status.homeSwitch){
            rc = p9018_axis_incr_move(axis, (vel < 0.0)? step_dist: -step_dist);
            if(!rc){
                rc = wait_axis(axis, 0);
            }
            if(rc){
                myPrint("fails to move backward[%d], rc: %d\n", axis, rc);
                return rc;
            }
        }
        Sleep(100);
        rc = p9018_axis_home(axis, vel, use_index);
        if(!rc){
            rc = wait_axis(axis, AXIS_HOMING_TIMEOUT);
        }
        if(rc){
            myPrint("fails to home[%d], rc: %d\n", axis, rc);
            return rc;
        }
    }

    if(rc){
        myPrint("fails to search home[%d], rc: %d\n", axis, rc);
        return rc;
    }

    //ڶβԭ㣬Ȼһξ룬ȻʹøٶȲԭ
    Sleep(100);
    rc = p9018_axis_incr_move(axis, (vel < 0.0)? step_dist: -step_dist);
    if(!rc){
        rc = wait_axis(axis, 0);
    }

    if(rc){
        myPrint("fails to move backward[%d], rc: %d\n", axis, rc);
        return rc;
    }
    Sleep(100);
    rc = p9018_axis_home(axis, vel * 0.1, use_index);
    if(!rc){
        rc = wait_axis(axis, AXIS_HOMING_TIMEOUT);
    }
    return rc;
}

int dump_axis(int axis)
{
    int rc = 0;
    MOT_AXIS_STAT axis_status;

    rc = p9018_axis_get_status(axis, &axis_status);
    if(!rc){
		myPrint("Axis[%d] pos: %.1f vel: %.1f, inpos: %d\n", axis, axis_status.cmdPos, axis_status.cmdVel, axis_status.inpos);
    }
    return rc;
}

/*! \fn double etime()
 *  \brief retrieves the high-resolution counter and convert it into seconds
 *  \param none
 *  \return timer value in seconds
 */
double etime()
{
	LARGE_INTEGER t, f;
	t.QuadPart = 0;
	f.QuadPart = 0;
	QueryPerformanceCounter(&t);
	QueryPerformanceFrequency(&f);
	if(f.QuadPart)
	{
		return (double)t.QuadPart/(double)f.QuadPart;
	}
	else
	{
		return 0.0;
	}
}

/*! \fn int myPrint(const char *_fmt, ...)
 *  \brief Ϣ޸Ĵͬ豸
 *
 *
 *  \param 
 *
 *  \return ʽַȣСʱʾ
 */
int myPrint(const char *_fmt, ...)
{
    int retval = 0;
    char buff[80+1];

    va_list args;
    va_start(args, _fmt);

    //retval = vsnprintf(buff, 80, _fmt, args);
    retval = vsnprintf_s(buff, sizeof(buff), _TRUNCATE, _fmt, args);    //safe version for VC++

    printf("%s", buff);         //output message to standard output
    //OutputDebugStringA(buff); //output message to system debugger

    va_end(args);
    return retval;
}
